
namespace muscle.iogateway {
  using muscle.support;
  using muscle.message;
  
  using System.IO;

  /// <summary>
  /// A gateway that converts to and from the 'standard' MUSCLE 
  /// flattened message byte stream.
  /// </summary>

  public class MessageIOGateway : AbstractMessageIOGateway {
    /// <summary>
    /// Reads from the input stream until a Message can be assembled 
    /// and returned.
    /// </summary>
    ///
    /// <param name="reader">the input stream from which to read</param>
    /// <exception cref="IOException"/>
    /// <exception cref="UnflattenFormatException"/>
    /// <returns>The next assembled Message</returns>
    ///
    public Message unflattenMessage(BinaryReader reader)
    {
       int numBytes = reader.ReadInt32();
       if (reader.ReadInt32() != _encoding) 
	 throw new IOException();

       Message pmsg = new Message();
       pmsg.unflatten(reader, numBytes);
       return pmsg;
    }
    
    /// <summary>
    /// Converts the given Message into bytes and sends it out the stream.
    /// </summary>
    /// <param name="writer">the data stream to send the converted bytes
    /// </param>
    /// <param name="msg">the message to convert</param>
    /// <exception cref="IOException"/>
    ///
    public void flattenMessage(BinaryWriter writer, Message msg)
    {
      writer.Write((int) msg.flattenedSize());
      writer.Write((int) _encoding);
      msg.flatten(writer);
    }
    
    /// 'Enc0'  ... our default (plain-vanilla) encoding 
    private int _encoding =  1164862256;  
  }
}
